/* Copyright (c) 2003 methodic[@libpcap.net], All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "orbs.h"
#include "global.h"

extern int optind;
extern char *optarg;

options_t *options = NULL;

int main(int argc, char *argv[]) {

	int i, opt;
	char ch, *hostlist=NULL;

	printf("::: orbs [version %d.%d] initializing..\n", RELMAJOR, RELMINOR);
	
	if(argc < 3) {
		usage(argv[0]);
	}
#ifdef DEBUG
	debug("parsing %d args", argc);
#endif

	/* malloc() memory for our structure */
	options = (options_t *)malloc(sizeof(options_t));
	/* set the defaults for our variables */
	/* see global.h for variable explanations */
	options->verbose = FALSE;
	options->ports = NULL;
	options->out_file = NULL;
	options->out_format = STDOUT;
	options->timeout = 5;
	options->parse_banners = TRUE;
	options->all_info = FALSE;

	while((opt = getopt(argc, argv, "vRap:r:f:o:t:")) != -1) {
		switch(opt) {
		case 'v':
			options->verbose = TRUE;
			break;
		case 'R':
			options->parse_banners = FALSE;
			break;
		case 'a':
			options->all_info = TRUE;
			break;
		case 'p':
			parse_ports(optarg);
			break;
		case 'r':
			hostlist = optarg;
			break;
		case 'f':
			/* find out logging format */
			ch = optarg[0];
			switch(ch) {
				case 't':
					options->out_format = TEXT;
					break;
				case 'h':
					options->out_format = HTML;
					break;
				default:
					printf("unknown output format: %c; reverting to stdout\n", ch);
					options->out_file = STDOUT;
			}
			break;
		case 'o':
			options->out_file = optarg;
			break;
		case 't':
			options->timeout = atoi(optarg);
			break;
		default:
			printf("unknown option: %c\n", opt);
		}
	}
	argc -= optind;
	argv += optind;

	/* minimal error checking */
	if(!argc && !hostlist) {
		fatal("couldn't find any host information");
	}
	if(options->ports == NULL) {
		fatal("couldn't find any port information");
	}
	if(options->out_file != NULL && options->out_format == 0) {
		fatal("need somewhere to log output to");
	}

	/* if our output is going to a file, we need to start it */
	if(options->out_format > 0) {
		output_handler(NULL, 0, NULL);
	}

	printf("::: orbs is running\n");

	/* check to see where we are getting our hosts to scan from */
	if(hostlist) {
		/* read in host information from file */
		hosts_from_file(hostlist);
	} else {
		/* we received the hosts as arguments on the command line */
#ifdef DEBUG
	debug("number of host arguments: %d", argc);
#endif
		for(i = 0; i < argc; i++) {
			//scan_host(argv[i]);
			parse_hosts(argv[i]);
		}
	}

	/* if our output is going to a file, we need to end it */
	if(options->out_format > 0) {
		output_handler(NULL, 1, NULL);
	}

	cleanup();

	printf("::: orbs completed\n");

	return(0);

}
