/* Copyright (c) 2003 methodic[@libpcap.net], All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include "orbs.h"
#include "global.h"

extern options_t *options;

void output_handler(char *host, int port, char *raw_banner) {

	int i=0;
	time_t curtime;
	char *strtime;
	FILE *fp;

	if(raw_banner != NULL) {
		/* see if we have to parse the banner out */
		if(raw_banner != NULL && options->parse_banners == TRUE) {
			raw_banner = parse_banner(raw_banner);
		}

		/* strip out ending newlines and return chars */
		while(raw_banner[strlen(raw_banner)-1] == '\r' || raw_banner[strlen(raw_banner)-1] == '\n') {
			raw_banner[strlen(raw_banner)-1] = '\0';
		}

		/* replace any other newlines and return chars with spaces */
		for(i = 0; i < strlen(raw_banner); i++) {
			if(raw_banner[i] == '\r' || raw_banner[i] == '\n') {
				raw_banner[i] = ' ';
			}
		}

	}

	if(options->out_format > 0) {
		/* attempt to open the log file */
		if( (fp = fopen(options->out_file, "a")) == NULL) {
			printf("cannot open %s for writing, switching to STDOUT\n", options->out_file);
			options->out_format = STDOUT;
		}

		switch(options->out_format) {
		case TEXT:
			if(host == NULL && port == 0 && raw_banner == NULL) {
				/* host is NULL and port is zero, start the text file */
				curtime = time(0);
				strtime = ctime(&curtime);
				strtime[strlen(strtime)-1] = '\0';
				fprintf(fp, "Scan started at %s by uid(%d)\n", strtime, getuid());
			} else if(host == NULL && port == 1 && raw_banner == NULL) {
				/* host is NULL and port is 1, end the text file */
				fprintf(fp, "\norbs [version %d.%d]\nhttp://libpcap.net/orbs/\n", RELMAJOR, RELMINOR);
			} else if(host != NULL && port == -1 && raw_banner == NULL) {
				/* if host isnt null and port is -1, host is unknown */
				fprintf(fp, "\n--[ unknown host: %s; skipping\n", host);
			} else if(host != NULL && port == 0 && raw_banner == NULL) {
				/* if host isnt null and port is zero, we write host info */
				fprintf(fp, "\n--[ banner information for %s\n", host);
			} else {
				fprintf(fp, "port[%d]: %s\n", port, raw_banner);
			}
			break;
		case HTML:
			if(host == NULL && port == 0) {
				/* host is NULL and port is zero, start the html file */
				curtime = time(0);
				strtime = ctime(&curtime);
				strtime[strlen(strtime)-1] = '\0';
				fprintf(fp, "<html>\n<head>\n<title>orbs results</title>\n</head>\n<body bgcolor=#ffffff>\nScan started at %s by uid(%d)<br>\n", strtime, getuid());
			} else if(host == NULL && port == 1) {
				/* host is NULL and port is 1, end the html file */
				fprintf(fp, "<br>\n<address>\norbs [version %d.%d]\n<br>\n<a href=http://libpcap.net/orbs/>http://libpcap.net/orbs/</a>\n</address></body>\n</html>\n", RELMAJOR, RELMINOR);
			} else if(host != NULL && port == -1 && raw_banner == NULL) {
				/* if host isnt null and port is -1, host is unknown */
				fprintf(fp, "<br><b>unknown host: <font color=blue>%s</font>; skipping</b><br>\n", host);
			} else if(host != NULL && port == 0 && raw_banner == NULL) {
				/* if host isnt null and port is zero, we write host info */
				fprintf(fp, "<br><b>banner information for <font color=blue>%s</font></b><br>\n", host);
			} else {
				fprintf(fp, "<b>port[<font color=red>%d</font>]</b>: %s<br>\n", port, raw_banner);
			}
			break;
		}
		/* close the log file */
		if(fp != NULL) {
			fclose(fp);
		}
	} else {
		/* no logging, print to console */
		if(host != NULL && port == -1 && raw_banner == NULL) {
			/* if host isnt null and port is -1, host is unknown */
			printf("--[ unknown host: %s; skipping\n", host);
		} else if(host != NULL && port == 0 && raw_banner == NULL) {
			/* if host isnt null and port is zero, we write host info */
			printf("--[ banner information for %s\n", host);
		} else {
			printf("port[%d]: %s\n", port, raw_banner);
		}
	}

}
