/* Copyright (c) 2003 methodic[@libpcap.net], All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "orbs.h"
#include "global.h"

extern options_t *options;

void parse_ports(char *ports) {

	unsigned int i=0, j=0, len=sizeof(int);
	char *comma=NULL, *dash=NULL;

#ifdef DEBUG
	debug("parsing \"%s\" as ports", ports);
#endif
	/* we allocate one byte to begin with */
	/* extra byte will be filled with a zero so we know when to stop */
	options->ports = (int *)malloc(len);

	/* search for a comma */
	comma = strtok(ports, ",");
	do {
		dash = strchr(comma, '-');
		if(dash != NULL) {
			/* we found a dash, which means we have a range of ports */
			/* find the first and last, and cycle through the rest */
			comma[strlen(comma) - strlen(dash)] = '\0';
			dash++;
			/* find out how many bytes we have to expand our array to */
			len += (sizeof(int) * (atoi(dash) - atoi(comma)) + 4);
			options->ports = (int *)realloc(options->ports, len);
			/* port range sanity checks */
			if(atoi(dash) <= atoi(comma)) {
				fatal("error in ports argument");
			} else if(atoi(comma) <= 0 || atoi(comma) > 65535) {
				fatal("error in ports argument");
			} else if(atoi(dash) <= 0 || atoi(dash) > 65535) {
				fatal("error in ports argument");
			}
			for(i = atoi(comma); i <= atoi(dash); i++, j++) {
#ifdef DEBUG
				debug("adding [%d] to ports array", i);
#endif
				options->ports[j] = i;
			}
		} else {
			/* no parsing needed here */
			if(atoi(comma) <= 0 || atoi(comma) > 65535) {
				fatal("error in ports argument");
			}
#ifdef DEBUG
			debug("adding [%d] to ports array", atoi(comma));
#endif
			/* increase array by 4 bytes */
			len += (sizeof(int));
			options->ports = (int *)realloc(options->ports, len);
			/* add the port to the array */
			options->ports[j] = atoi(comma);
			j++;
		}
		comma = strtok(NULL, ",");
	}while(comma != NULL);

#ifdef DEBUG
	debug("malloc()'d %d bytes for ports array", len);
#endif

	/* add a -1 to the end of our array, signifying the end */
	options->ports[j] = -1;

}
