/* Copyright (c) 2003 methodic[@libpcap.net], All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>
#include "orbs.h"
#include "global.h"

extern options_t *options;

void scan_host(char *host) {

	int i=0, banner=0, ret, sockfd;
	char raw_banner[LEN];

	struct itimerval itv; /* connect() timer structure */
	struct sigaction sig; /* signal structure */

	struct timeval timeout;
	fd_set fds;

	struct sockaddr_in addr; /* socket address structure */
	struct hostent *h_ent; /* host info structure */

	/* zero out our socket structure */
	bzero(&addr, sizeof(addr));

	/* see if we can look up the host */
	h_ent = gethostbyname(host);
	if(h_ent != NULL) {
		memcpy((char *)&addr.sin_addr, h_ent->h_addr, h_ent->h_length);
	} else {
		addr.sin_addr.s_addr = inet_addr(host);
		if(addr.sin_addr.s_addr == -1) {
			if(options->verbose && options->out_format > 0) {
				printf("--[ unknown host: %s; skipping\n", host);
			}
			output_handler(host, -1, NULL);
			return;
		}
	}

	if(options->verbose) {
		printf("--[ scanning %s\n", host);
	}

	/* write the host info before the port info */
	if(options->all_info == TRUE) {
		output_handler(host, 0, NULL);
	}

	while(options->ports[i] > 0) {
#ifdef DEBUG
		debug("trying port [%d] on %s", options->ports[i], host);
#endif
		/* create a socket */
		sockfd = socket(AF_INET, SOCK_STREAM, 0);
		if(sockfd == -1) {
			fatal("could not create socket");
		}
		/* plug in protocol and port info */
		addr.sin_family = AF_INET;
		addr.sin_port = htons(options->ports[i]);

		/* create timeout values for connect() */
		itv.it_value.tv_sec = options->timeout;
		itv.it_value.tv_usec = 0;
		itv.it_interval = itv.it_value;
		setitimer(ITIMER_REAL, &itv, NULL);

		/* setup signals */
		sig.sa_handler = catch_signal;
		sig.sa_flags = 0;
		sigemptyset(&sig.sa_mask);
		sigaction(SIGALRM, &sig, 0);

		/* connect to the host/port */
		ret = connect(sockfd, (struct sockaddr *)&addr, sizeof(addr));
		if(ret == 0) {
			/* connected, clear connect() timeout values */
			itv.it_value.tv_sec = 0;
			itv.it_value.tv_usec = 0;
			itv.it_interval = itv.it_value;
			setitimer(ITIMER_REAL, &itv, NULL);

#ifdef DEBUG
			debug("connect(%s) success to port [%d]", host, options->ports[i]);
#endif

			/* see if we have a port hook */
			do_port_hook(options->ports[i], sockfd);

			/* set timeout values for recv() */
			timeout.tv_sec = options->timeout;
			timeout.tv_usec = 0;
			FD_ZERO(&fds);
			FD_SET(sockfd, &fds);
			fcntl(sockfd, F_SETFL, O_NONBLOCK);
			select(sockfd+1, &fds, NULL, NULL, &timeout);

			bzero(raw_banner, LEN);
			ret = recv(sockfd, raw_banner, (LEN-1), 0);
			if(ret > 0) {
			/* we got some sort of data.. */
				if(banner == 0 && options->all_info == FALSE) {
					output_handler(host, 0, NULL);
					banner++;
				}
				if(options->ports[i] == 23) {
					/* telnet server, special case */
					get_telnet_banner(sockfd, host, options->ports[i], raw_banner);
				} else {
					output_handler(host, options->ports[i], raw_banner);
				}
			} else if(ret == -1) {
				/* timed out */
				if(options->all_info == TRUE) {
					output_handler(host, options->ports[i], "port timed out");
				}
			} else {
				/* no data? */
				if(options->all_info == TRUE) {
					output_handler(host, options->ports[i], "no data on port");
				}
			}
		} else {
#ifdef DEBUG
			debug("connect(%s) failed to port [%d]", host, options->ports[i]);
#endif
			if(options->all_info == TRUE) {
				output_handler(host, options->ports[i], "couldn't connect to port");
			}
		}
		close(sockfd);
		i++;
	}

}
